{--------------------------------------------------------------------------------
* Description : TscSystemInfo
  - Computername
  - Username
  - CPU Speed
  - Operating System Version
  - System Memory
  - System Resources
  - Application versions (Delphi, Office)
* Dates : January 2001 - December 2007
* Version : 1.6

* Author : Stefan Cruysberghs
* Email : stefancr@scip.be
* Website : http://www.scip.be
  Visit SCIP.be for other components, tools, articles, ...
--------------------------------------------------------------------------------
* $Archive: /Delphi/Delphi2007/Component Library/SC_Public/scSystemInfo.pas $
* $Modtime: 8/12/07 11:07 $
--------------------------------------------------------------------------------
* This component is free of charge.
* The author doesn't give a warranty for error free running
  of this component and he doesn't give any support.
* Suggestions and bugs can be send by email.
--------------------------------------------------------------------------------
Installation
* Install the TscSystemInfo component by adding the unit scSystemInfo.pas
  to a package. Remove the DCR file in the package. Compile and Install the package.
* The TscSystemInfo component can be found in the tabsheet 'SC'
  of the component palette.
--------------------------------------------------------------------------------
* History
Version 1.6 (8/12/2007)
- Implementation of GlobalMemoryStatusEx function to get memory sizes larger then 2Gb
- Defined 2 compiler directives WIN_OLD and WIN_NEW
Version 1.5 (20/05/2007)
- Added version recognition for Delphi 2007
Version 1.4 (6/04/2006)
- Added version recognition for Delphi 2006, Office 2007, Windows Vista and Windows Server 2003
Version 1.3 (2/12/2004)
- Added application version (Office and Delphi)
Version 1.2 (1/8/2002)
- Added CPUSpeed
- Added extra info abouw Windows versioning
- Improvements refreshing information
Version 1.1 (20/02/2002)
- Added extra info abouw Windows versioning
Version 1.0 (10/06/2001)
- First version
--------------------------------------------------------------------------------
* Compiler directives
- Uncomment one of these 2 compiler directives
- WIN_OLD : Windows 95, 98, ME, NT
- WIN_NEW : Windows 2000, XP, Vista, 2003, 2008
- Only WIN_NEW supports GlobalMemoryStatusEx function which can retrieve memory sizes largen then 2Gb
--------------------------------------------------------------------------------}
// {$DEFINE WIN_OLD}
 {$DEFINE WIN_NEW}

unit scSystemInfo;

interface

uses
  Windows, Classes, SysUtils;

type
  DWORDLONG       = int64;
  _MEMORYSTATUSEX = packed record
    dwLength                : DWORD;
    dwMemoryLoad            : DWORD;
    ullTotalPhys            : DWORDLONG;
    ullAvailPhys            : DWORDLONG ;
    ullTotalPageFile        : DWORDLONG;
    ullAvailPageFile        : DWORDLONG;
    ullTotalVirtual         : DWORDLONG;
    ullAvailVirtual         : DWORDLONG;
    ullAvailExtendedVirtual : DWORDLONG;
  end;
  {$EXTERNALSYM _MEMORYSTATUSEX}
  TMemoryStatusEx = _MEMORYSTATUSEX;
  MEMORYSTATUSEX = _MEMORYSTATUSEX;
  {$EXTERNALSYM MEMORYSTATUSEX}

  function GlobalMemoryStatusEx(var lpBuffer:TMEMORYSTATUSEX) : Boolean; stdcall;
  {$EXTERNALSYM GlobalMemoryStatusEx}

type
//------------------------------------------------------------------------------
  TscSysMemory = class(TPersistent)
  private
    FIntMemoryFree    : Byte;
    FIntMemoryUsed    : Byte;
    FIntTotalPhys     : Integer;
    FIntAvailPhys     : Integer;
    FIntTotalPageFile : Integer;
    FIntAvailPageFile : Integer;
    FIntTotalVirtual  : Integer;
    FIntAvailVirtual  : Integer;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure   RefreshInfo;
  published
    // Percentages
    property MemoryFree    : Byte    read FIntMemoryFree    write FIntMemoryFree stored False;
    property MemoryUsed    : Byte    read FIntMemoryUsed    write FIntMemoryUsed stored False;
    // In Bytes
    property TotalPhys     : Integer read FIntTotalPhys     write FIntTotalPhys stored False;
    property AvailPhys     : Integer read FIntAvailPhys     write FIntAvailPhys stored False;
    property TotalPageFile : Integer read FIntTotalPageFile write FIntTotalPageFile stored False;
    property AvailPageFile : Integer read FIntAvailPageFile write FIntAvailPageFile stored False;
    property TotalVirtual  : Integer read FIntTotalVirtual  write FIntTotalVirtual stored False;
    property AvailVirtual  : Integer read FIntAvailVirtual  write FIntAvailVirtual stored False;
  end;

//------------------------------------------------------------------------------
  TscSysResources = class(TPersistent)
  private
    FIntSystemResources : Byte;
    FIntGDIResources : Byte;
    FIntUserResources : Byte;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure   RefreshInfo;
  published
    property SystemResources : Byte read FIntSystemResources write FIntSystemResources stored False;
    property GDIResources    : Byte read FIntGDIResources    write FIntGDIResources stored False;
    property UserResources   : Byte read FIntUserResources   write FIntUserResources stored False;
  end;

//------------------------------------------------------------------------------
  TPlatform = (pfUnknown,pfWin32,pfWin95,pfWin95OSR2,pfWin98,pfWin98SE,pfWinME,pfWinNT35,pfWinNT4,pfWin2000,pfWinXP,pfWinServer2003,pfWinVista);
  TOSKernel = (ok95Kern, okNTKern);

  TscOperatingSystem = class(TPersistent)
  private
    FIntMajorVersion : Integer;
    FIntMinorVersion : Integer;
    FIntBuildNumber  : Integer;
    FIntPlatFormID   : Integer;
    FStrCSDVersion   : String;
    FPlatForm        : TPlatform;
    FOSKernel        : TOSKernel;
    FStrDescription  : String;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure   RefreshInfo;
  published
    // Platform = pfUnknown,pfWin32,pfWin95,pfWin95OSR2,pfWin98,pfWin98SE,pfWinME,pfWinNT35,pfWinNT4,pfWin2000,pfWinXP,pfWinServer2003,pfWinVista
    property Platform     : TPlatform  read FPlatform        write FPlatform stored False;
    property MajorVersion : Integer    read FIntMajorVersion write FIntMajorVersion stored False;
    property MinorVersion : Integer    read FIntMinorVersion write FIntMinorVersion stored False;
    property BuildNumber  : Integer    read FIntBuildNumber  write FIntBuildNumber stored False;
    property PlatFormID   : Integer    read FIntPlatFormID   write FIntPlatFormID stored False;
    // Extra info about service packs
    property CSDVersion   : String     read FStrCSDVersion   write FStrCSDVersion stored False;
    // Kernel : 95 or NT
    property OSKernel     : TOSKernel  read FOSKernel        write FOSKernel stored False;
    property Description  : String     read FStrDescription  write FStrDescription stored False;
  end;

  TscAppVersions = class(TPersistent)
  private
    FIntDelphiVersion: Integer;
    FIntOfficeVersion: Integer;
    function GetDelphiVersion: Integer;
    function GetOfficeVersion: Integer;
  published
    property OfficeVersion : Integer read GetOfficeVersion write FIntOfficeVersion;
    property DelphiVersion : Integer read GetDelphiVersion write FIntDelphiVersion;
  end;

//------------------------------------------------------------------------------
  TscSystemInfo = class(TComponent)
  private
    FStrComputerName : String;
    FStrUserName     : String;
    FFltCPUSpeed     : Double;
    FSysMemory       : TscSysMemory;
    FSystemResources : TscSysResources;
    FOperatingSystem : TscOperatingSystem;
    FAppVersions     : TscAppVersions;

    FBlnAutoRefresh  : Boolean;
    BlnResourceInfoAvailable : Boolean;

    procedure SetAutoRefresh(Value: Boolean);
    function GetComputer : String;
    function GetUser : String;
    function GetCPUSpeed : Double;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   RefreshInfo;
  published
    // Computer name
    property ComputerName    : String             read FStrComputerName write FStrComputerName stored False;
    // Current user
    property UserName        : String             read FStrUserName     write FStrUserName stored False;
    // Real CPU speed
    property CPUSpeed        : Double             read FFltCPUSpeed     write FFltCPUSpeed stored False;
    // Memory
    property SysMemory       : TscSysMemory       read FSysMemory       write FSysMemory;
    // Resources
    // In WinNT/2000/XP there are no limits on resources
    // So this information is only available in Win95/98/Me
    property SystemResources : TscSysResources    read FSystemResources write FSystemResources;
    // Operating system
    property OperatingSystem : TscOperatingSystem read FOperatingSystem write FOperatingSystem;
    property AutoRefresh     : Boolean            read FBlnAutoRefresh  write SetAutoRefresh default False;
    property ResourceInfoAvailable : Boolean      read BlnResourceInfoAvailable stored False;
    // Application versions
    property AppVersion      : TscAppVersions     read FAppVersions     write FAppVersions;
  end;

procedure Register;

implementation

{$R scSystemInfo.dcr}

uses Registry;

const
  CONST_WIN32 = 0;
  CONST_WIN95 = 1;
  CONST_WINNT = 2;

  CONST_SYSRES = 0;
  CONST_GDIRES = 1;
  CONST_USERES = 2;

function GlobalMemoryStatusEx; external 'kernel32' name 'GlobalMemoryStatusEx';

procedure Register;
begin
  RegisterComponents('SC', [TscSystemInfo]);
end;

//------------------------------------------------------------------------------
// Memory
//------------------------------------------------------------------------------
constructor TscSysMemory.Create;
begin
  inherited Create;
  FIntMemoryUsed    := 0;
  FIntMemoryFree    := 100;
  FIntTotalPhys     := 0;
  FIntAvailPhys     := 0;
  FIntTotalPageFile := 0;
  FIntAvailPageFile := 0;
  FIntTotalVirtual  := 0;
  FIntAvailVirtual  := 0;
end;

//------------------------------------------------------------------------------
destructor TscSysMemory.Destroy;
begin
  inherited Destroy;
end;

//------------------------------------------------------------------------------
procedure TscSysMemory.RefreshInfo;
var
  {$IFDEF WIN_NEW}
  MyMemoryEx : TMemoryStatusEx;
  {$ELSE}
  MyMemory: TMemoryStatus;
  {$ENDIF}
begin
  {$IFDEF WIN_NEW}
  FIntMemoryUsed    := 0;
  FIntMemoryFree    := 100;
  FIntTotalPhys     := 0;
  FIntAvailPhys     := 0;
  FIntTotalPageFile := 0;
  FIntAvailPageFile := 0;
  FIntTotalVirtual  := 0;
  FIntAvailVirtual  := 0;

  MyMemoryEx.dwLength:= sizeof(MyMemoryEx);

  if GlobalMemoryStatusEx(MyMemoryEx) then
  begin
    FIntMemoryUsed    := MyMemoryEx.dwMemoryLoad;
    FIntMemoryFree    := (100 - MyMemoryEx.dwMemoryLoad);
    FIntTotalPhys     := MyMemoryEx.ullTotalPhys;
    FIntAvailPhys     := MyMemoryEx.ullAvailPhys;
    FIntTotalVirtual  := MyMemoryEx.ullTotalVirtual;
    FIntAvailVirtual  := MyMemoryEx.ullAvailVirtual;
    FIntTotalPageFile := MyMemoryEx.ullTotalPageFile;
    FIntAvailPageFile := MyMemoryEx.ullAvailPageFile;
  end;
  {$ELSE}
  try
    MyMemory.dwLength := SizeOf(MyMemory);
    GlobalMemoryStatus(MyMemory);
    with MyMemory do
    begin
      FIntMemoryUsed    := dwMemoryLoad;
      FIntMemoryFree    := (100 - dwMemoryLoad);
      FIntTotalPhys     := dwTotalPhys;
      FIntAvailPhys     := dwAvailPhys;
      FIntTotalVirtual  := dwTotalVirtual;
      FIntAvailVirtual  := dwAvailVirtual;
      FIntTotalPageFile := dwTotalPageFile;
      FIntAvailPageFile := dwAvailPageFile;
    end;
  except
    FIntMemoryUsed    := 0;
    FIntMemoryFree    := 100;
    FIntTotalPhys     := 0;
    FIntAvailPhys     := 0;
    FIntTotalPageFile := 0;
    FIntAvailPageFile := 0;
    FIntTotalVirtual  := 0;
    FIntAvailVirtual  := 0;
  end;
  {$ENDIF}
end;

//------------------------------------------------------------------------------
// Resources
//------------------------------------------------------------------------------
// Absolete functions in Windows NT but still available, so they don't need to
// be created dynamicly
//------------------------------------------------------------------------------
function  LoadLibrary16(LibraryName: PChar): THandle;                     stdcall; external kernel32 index 35;
procedure FreeLibrary16(HInstance: THandle);                              stdcall; external kernel32 index 36;
function  GetProcAddress16(Hinstance: THandle; ProcName: PChar): Pointer; stdcall; external kernel32 index 37;

//------------------------------------------------------------------------------
constructor TscSysResources.Create;
begin
  inherited Create;
  FIntSystemResources := 100;
  FIntGDIResources := 100;
  FIntUserResources := 100;
end;

//------------------------------------------------------------------------------
destructor TscSysResources.Destroy;
begin
  inherited Destroy;
end;

//------------------------------------------------------------------------------
// In WinNT/2000/XP there are no limits on resources
// So this information is only available in Win95/98/Me
//------------------------------------------------------------------------------
procedure TscSysResources.RefreshInfo;

  function GetFreeResources(Value: Word): Word;
  var
    UserHandle, KernelHandle: THandle;
    GetFreeSystemResources:   Pointer;
    Thunk:                    Pointer;
    Thunks: array[0..32] of   Word;
  begin
    Result := 100;

    UserHandle   := LoadLibrary16('USER.EXE');
    KernelHandle := LoadLibrary('KERNEL32.DLL');
    Thunks[0]    := UserHandle;

    if (UserHandle > 0) and (KernelHandle > 0) then
    begin
      GetFreeSystemResources := GetProcAddress16(UserHandle, 'GetFreeSystemResources');
      Thunk                  := GetProcAddress(KernelHandle, 'QT_Thunk');
      if (Assigned(GetFreeSystemResources)) and (Assigned(Thunk))then
      begin
        asm
          PUSH     Value
          MOV EDX, GetFreeSystemResources
          CALL     Thunk
          MOV      Result, AX
        end;
      end;
      FreeLibrary16(UserHandle);
      FreeLibrary(KernelHandle);
    end;
  end;

begin
  try
    FIntSystemResources := GetFreeResources(CONST_SYSRES);
    FIntGDIResources := GetFreeResources(CONST_GDIRES);
    FIntUserResources := GetFreeResources(CONST_USERES);
  except
    FIntSystemResources := 100;
    FIntGDIResources := 100;
    FIntUserResources := 100;
  end;
end;

//------------------------------------------------------------------------------
// Operating Sytem
//------------------------------------------------------------------------------
constructor TscOperatingSystem.Create;
begin
  inherited Create;
  FIntMajorVersion := 0;
  FIntMinorVersion := 0;
  FIntBuildNumber  := 0;
  FIntPlatFormID   := 0;
  FStrCSDVersion   := '';
end;

//------------------------------------------------------------------------------
destructor TscOperatingSystem.Destroy;
begin
  inherited Destroy;
end;

{------------------------------------------------------------------------------
  Windows 95 retail, OEM    4.00.950                      7/11/95
  Windows 95 retail SP1     4.00.950A                     7/11/95-12/31/95
  OEM Service Release 2     4.00.1111* (4.00.950B)        8/24/96
  OEM Service Release 2.1   4.03.1212-1214* (4.00.950B)   8/24/96-8/27/97
  OEM Service Release 2.5   4.03.1214* (4.00.950C)        8/24/96-11/18/97
  Windows 98 retail, OEM    4.10.1998                     5/11/98
  Windows 98 Second Edition 4.10.2222A                    4/23/99
  Windows Millennium        4.90.3000
  Windows 2000 	            5.00
  Windows XP Beta 2         5.01.2462
  Windows XP                5.01.2600
  Windows Server 2003 	    5.02
  Windows Server 2003 R2 	  5.02
  Windows Vista 	          6.00
  Windows Server 2008       6.00
------------------------------------------------------------------------------}
procedure TscOperatingSystem.RefreshInfo;
var
  CurrentOS : TOSVersionInfo;
begin
  try
    //Operating System
    CurrentOS.dwOSVersionInfoSize := SizeOf(CurrentOS);
    GetVersionEx(CurrentOS);
    with CurrentOS do
    begin
      FIntMajorVersion := dwMajorVersion;
      FIntMinorVersion := dwMinorVersion;
      FIntBuildNumber  := dwBuildNumber;
      FIntPlatFormID   := dwPlatformId;
      FStrCSDVersion   := szCSDVersion;

      case FIntPlatFormID of
        CONST_WIN32 :
          begin
            FPlatform := pfWin32; // Win32 on Win3.11
            FStrDescription := 'Win32';
          end;
        CONST_WIN95 :
          begin
            if FIntMajorVersion = 4 then
            begin
              if FIntMinorVersion < 10 then
              begin
                if Trim(FStrCSDVersion) = 'B' then
                begin
                  FPlatform := pfWin95OSR2;
                  FStrDescription := 'Windows 95 OSR2';
                end
                else
                begin
                  FPlatform := pfWin95;
                  FStrDescription := 'Windows 95';
                end;
              end
              else
              begin
                if FIntMinorVersion = 10 then
                begin
                  if Trim(FStrCSDVersion) = 'B' then
                  begin
                    FPlatform := pfWin98SE;
                    FStrDescription := 'Windows 98 SE';
                  end
                  else
                  begin
                    FPlatform := pfWin98;
                    FStrDescription := 'Windows 98';
                  end
                end
                else //90
                begin
                  FPlatform := pfWinME;
                  FStrDescription := 'Windows ME';
                end;
              end;
            end;
          end;
        CONST_WINNT :
          begin
            if FIntMajorVersion = 3 then
            begin
              FPlatform := pfWinNT35;
              FStrDescription := 'Windows NT 3.5';
            end
            else
            begin
              if FIntMajorVersion = 4 then
              begin
                FPlatform := pfWinNT4;
                FStrDescription := 'Windows NT 4.0';
              end
              else
              begin
                if FIntMajorVersion = 5 then
                begin
                  if FIntMinorVersion = 0 then
                  begin
                    FPlatform := pfWin2000;
                    FStrDescription := 'Windows 2000';
                  end
                  else if FIntMinorVersion = 1 then
                  begin
                    FPlatform := pfWinXP;
                    FStrDescription := 'Windows XP';
                  end
                  else if FIntMinorVersion = 2 then
                  begin
                    FPlatform := pfWinServer2003;
                    FStrDescription := 'Windows Server 2003';
                  end;
                end
                else if FIntMajorVersion = 6 then
                begin
                  if FIntMinorVersion = 0 then
                  begin
                    FPlatform := pfWinVista;
                    FStrDescription := 'Windows Vista';
                  end;
                end;
              end;
            end;
          end;
      end;
    end;

    if FPlatForm in [pfWin95,pfWin95OSR2,pfWin98,pfWin98SE,pfWinME] then
      FOSKernel := ok95Kern
    else
      FOSKernel := okNTKern;
  except
    FIntMajorVersion := 0;
    FIntMinorVersion := 0;
    FIntBuildNumber  := 0;
    FIntPlatFormID   := 0;
    FStrCSDVersion   := '';
    FPlatform        := pfUnknown;
  end;
end;

//------------------------------------------------------------------------------
function TscAppVersions.GetOfficeVersion : Integer;
var
  Regist : TRegistry;
begin
  if FIntOfficeVersion = 0 then
  begin
    Result := 0;
    Regist := TRegistry.Create;
    try
      Regist.RootKey := Windows.HKEY_CURRENT_USER;
      if Regist.OpenKey('\Software\Microsoft\Office\12.0',false) then
        Result := 12 //2007
      else if Regist.OpenKey('\Software\Microsoft\Office\11.0',false) then
        Result := 11 //2003
      else if Regist.OpenKey('\Software\Microsoft\Office\10.0',false) then
        Result := 10 //XP
      else if Regist.OpenKey('\Software\Microsoft\Office\9.0',false) then
        Result := 9 //2000
      else if Regist.OpenKey('\Software\Microsoft\Office\8.0',false) then
        Result := 8 //97
      else if Regist.OpenKey('\Software\Microsoft\Office\7.0',false) then
        Result := 7; //95
      Regist.CloseKey;
    finally
      Regist.Free;
    end;
    FIntOfficeVersion := Result;
  end
  else
    Result := FIntOfficeVersion;
end;

//------------------------------------------------------------------------------
function TscAppVersions.GetDelphiVersion : Integer;
var
  Regist : TRegistry;
begin
  if FIntDelphiVersion = 0 then
  begin
    Result := 0;
    Regist := TRegistry.Create;
    try
      Regist.RootKey := Windows.HKEY_CURRENT_USER;
      if Regist.OpenKey('\Software\Borland\BDS\5.0',false) then
        Result := 11 //2007
      else if Regist.OpenKey('\Software\Borland\BDS\4.0',false) then
        Result := 10 //2006
      else if Regist.OpenKey('\Software\Borland\BDS\3.0',false) then
        Result := 9 //2005
      else if Regist.OpenKey('\Software\Borland\BDS\2.0',false) then
        Result := 8
      else if Regist.OpenKey('\Software\Borland\Delphi\7.0',false) then
        Result := 7
      else if Regist.OpenKey('\Software\Borland\Delphi\6.0',false) then
        Result := 6
      else if Regist.OpenKey('\Software\Borland\Delphi\5.0',false) then
        Result := 5
      else if Regist.OpenKey('\Software\Borland\Delphi\4.0',false) then
        Result := 4
      else if Regist.OpenKey('\Software\Borland\Delphi\3.0',false) then
        Result := 3
      else if Regist.OpenKey('\Software\Borland\Delphi\2.0',false) then
        Result := 2;
      Regist.CloseKey;
    finally
      Regist.Free;
    end;
    FIntDelphiVersion := Result;
  end
  else
    Result := FIntDelphiVersion;
end;

//------------------------------------------------------------------------------
// System Info
//------------------------------------------------------------------------------
constructor TscSystemInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSysMemory       := TscSysMemory.Create;
  FSystemResources := TscSysResources.Create;
  FOperatingSystem := TscOperatingSystem.Create;
  FAppVersions     := TscAppVersions.Create;
  FBlnAutoRefresh  := False;

  // Only refresh this info once
  BlnResourceInfoAvailable := (FOperatingSystem.FOSKernel = ok95Kern);
  FOperatingSystem.RefreshInfo;
end;

//------------------------------------------------------------------------------
destructor TscSystemInfo.Destroy;
begin
  FSysMemory.Free;
  FSystemResources.Free;
  FOperatingSystem.Free;
  FAppVersions.Free;
  inherited Destroy;
end;

//------------------------------------------------------------------------------
procedure TscSystemInfo.SetAutoRefresh(Value: Boolean);
begin
  FBlnAutoRefresh := Value;
  if Value then
    RefreshInfo;
end;

//------------------------------------------------------------------------------
procedure TscSystemInfo.RefreshInfo;
begin
  //Memory and Resources
  FSysMemory.RefreshInfo;
  if BlnResourceInfoAvailable then
    FSystemResources.RefreshInfo;
  FStrComputerName := GetComputer;
  FStrUserName :=GetUser;
  FFltCPUSpeed := GetCPUSpeed;
  FAppVersions.GetOfficeVersion;
  FAppVersions.GetDelphiVersion;
end;

//------------------------------------------------------------------------------
function TscSystemInfo.GetUser : String;
var
  ChrUser : array[0..127] of Char;
  IntSize : DWord;
begin
  IntSize := 128;
  GetUserName(ChrUser, IntSize);
  Result := StrPas(ChrUser);
end;

//------------------------------------------------------------------------------
function TscSystemInfo.GetComputer : String;
var
  ChrComputer : array[0..127] of Char;
  IntSize : DWord;
begin
  IntSize := 128;
  GetComputerName(ChrComputer, IntSize);
  Result := StrPas(ChrComputer);
end;

//------------------------------------------------------------------------------
function TscSystemInfo.GetCPUSpeed : Double;
const
  IntDelayTime = 500; // time in ms
var 
  TimerHi, TimerLo: DWORD;
  IntPriorityClass, IntPriority: Integer;
begin 
  IntPriorityClass := GetPriorityClass(GetCurrentProcess);
  IntPriority := GetThreadPriority(GetCurrentThread);

  SetPriorityClass(GetCurrentProcess, REALTIME_PRIORITY_CLASS); 
  SetThreadPriority(GetCurrentThread, THREAD_PRIORITY_TIME_CRITICAL); 

  Sleep(10);
  asm 
    dw 310Fh
    mov TimerLo, eax
    mov TimerHi, edx
  end; 
  Sleep(IntDelayTime); 
  asm
    dw 310Fh
    sub eax, TimerLo 
    sbb edx, TimerHi 
    mov TimerLo, eax
    mov TimerHi, edx 
  end;

  SetThreadPriority(GetCurrentThread, IntPriority);
  SetPriorityClass(GetCurrentProcess, IntPriorityClass);

  Result := TimerLo / (1000.0 * IntDelayTime);
end;

end.
